@@ -8,18 +8,19 @@ from office365.directory.groups.group_setting_template import GroupSettingTempla
 from office365.directory.groups.group_collection import GroupCollection
 from office365.directory.identities.identity_container import IdentityContainer
 from office365.directory.identities.identity_provider import IdentityProvider
+from office365.directory.organizations.org_contact import OrgContact
 from office365.directory.organizations.organization import Organization
 from office365.directory.applications.service_principal import ServicePrincipal
 from office365.directory.licenses.subscribed_sku import SubscribedSku
-from office365.directory.reports.reports_root import ReportsRoot
-from office365.directory.subscription import Subscription
+from office365.outlook.calendar.place import Place
+from office365.reports.report_root import ReportRoot
+from office365.directory.subscriptions.subscription import Subscription
 from office365.directory.users.user import User
 from office365.directory.users.user_collection import UserCollection
 from office365.entity_collection import EntityCollection
 from office365.onedrive.drives.drive import Drive
 from office365.onedrive.shares.shares_collection import SharesCollection
 from office365.onedrive.sites.sites_with_root import SitesWithRoot
-from office365.outlook.contacts.contact import Contact
 from office365.planner.planner import Planner
 from office365.runtime.auth.token_response import TokenResponse
 from office365.runtime.client_runtime_context import ClientRuntimeContext
@@ -32,6 +33,7 @@ from office365.runtime.queries.batch_query import BatchQuery
 from office365.runtime.queries.delete_entity_query import DeleteEntityQuery
 from office365.runtime.queries.update_entity_query import UpdateEntityQuery
 from office365.runtime.resource_path import ResourcePath
+from office365.search.search_entity import SearchEntity
 from office365.teams.chats.chat import Chat
 from office365.communications.cloud_communications import CloudCommunications
 from office365.teams.team_collection import TeamCollection
@@ -88,12 +90,13 @@ class GraphClient(ClientRuntimeContext):
 
     def authenticate_request(self, request):
         """
+        Authenticate request
 
         :type request: RequestOptions
         """
         token_json = self._acquire_token_callback()
         token = TokenResponse.from_json(token_json)
-        request.set_header('Authorization', 'Bearer {0}'.format(token.accessToken))
+        request.ensure_header('Authorization', 'Bearer {0}'.format(token.accessToken))
 
     @property
     def me(self):
@@ -147,8 +150,8 @@ class GraphClient(ClientRuntimeContext):
 
     @property
     def contacts(self):
-        """o get all the contacts in a user's mailbox"""
-        return EntityCollection(self, Contact, ResourcePath("contacts"))
+        """Get the list of organizational contacts for this organization."""
+        return EntityCollection(self, OrgContact, ResourcePath("contacts"))
 
     @property
     def directory(self):
@@ -210,12 +213,19 @@ class GraphClient(ClientRuntimeContext):
         """
         return AuditLogRoot(self, ResourcePath("auditLogs"))
 
+    @property
+    def places(self):
+        """
+        Get all places in a tenant
+        """
+        return EntityCollection(self, Place, ResourcePath("places"))
+
     @property
     def reports(self):
         """
         The resource that represents an instance of History Reports.
         """
-        return ReportsRoot(self, ResourcePath("reports"))
+        return ReportRoot(self, ResourcePath("reports"))
 
     @property
     def teams_templates(self):
@@ -231,3 +241,10 @@ class GraphClient(ClientRuntimeContext):
         It returns a singleton planner resource. It doesn't contain any usable properties.
         """
         return Planner(self, ResourcePath("planner"))
+
+    @property
+    def search(self):
+        """
+        The search endpoint is the entry point for Microsoft Search API to query data.
+        """
+        return SearchEntity(self, ResourcePath("search"))
