@@ -1,4 +1,4 @@
-from office365.directory.subscription import Subscription
+from office365.directory.subscriptions.subscription import Subscription
 from office365.onedrive.driveitems.audio import Audio
 from office365.onedrive.driveitems.geo_coordinates import GeoCoordinates
 from office365.onedrive.driveitems.image import Image
@@ -38,7 +38,7 @@ class DriveItem(BaseItem):
 
     def get_by_path(self, url_path):
         """
-        Retrieve DriveItem by path
+        Retrieve DriveItem by server relative path
 
         :type url_path: str
         """
