@@ -20,3 +20,11 @@ class ContactFolder(Entity):
         return self.properties.get('childFolders',
                                    EntityCollection(self.context, ContactFolder,
                                                     ResourcePath("childFolders", self.resource_path)))
+
+    def get_property(self, name, default_value=None):
+        if default_value is None:
+            property_mapping = {
+                "childFolders": self.child_folders
+            }
+            default_value = property_mapping.get(name, None)
+        return super(ContactFolder, self).get_property(name, default_value)
