@@ -1,8 +1,9 @@
 import copy
 
 from office365.runtime.client_value import ClientValue
-from office365.runtime.odata.json_light_format import JsonLightFormat
-from office365.runtime.odata.odata_query_options import QueryOptions
+from office365.runtime.odata.v3.json_light_format import JsonLightFormat
+from office365.runtime.odata.odata_json_format import ODataJsonFormat
+from office365.runtime.odata.query_options import QueryOptions
 
 
 class ClientObject(object):
@@ -17,7 +18,7 @@ class ClientObject(object):
         :type namespace: str
         """
         self._properties = {}
-        self._changed_properties = []
+        self._properties_metadata = {}
         self._entity_type_name = None
         self._query_options = QueryOptions()
         self._parent_collection = parent_collection
@@ -25,14 +26,25 @@ class ClientObject(object):
         self._resource_path = resource_path
         self._namespace = namespace
 
+    def set_metadata(self, name, group, value):
+        if name not in self._properties_metadata:
+            self._properties_metadata[name] = {}
+        self._properties_metadata[name][group] = value
+
+    def get_metadata(self, name, group, default_value=None):
+        return self._properties_metadata.get(name, {}).get(group, default_value)
+
     def clear(self):
-        self._changed_properties = []
+        self._properties_metadata = {}
 
     def execute_query(self):
         self.context.execute_query()
         return self
 
-    def execute_query_retry(self, max_retry=5, timeout_secs=5, success_callback=None, failure_callback=None):
+    def execute_query_retry(self, max_retry=5,
+                            timeout_secs=5,
+                            success_callback=None,
+                            failure_callback=None):
         self.context.execute_query_retry(max_retry=max_retry,
                                          timeout_secs=timeout_secs,
                                          success_callback=success_callback,
@@ -77,14 +89,17 @@ class ClientObject(object):
             return
         self._parent_collection.remove_child(self)
 
-    def get_property(self, name):
+    def get_property(self, name, default_value=None):
         """
         Gets property value
 
         :param str name: property name
+        :param any default_value: property value
         """
-        normalized_name = name[0].lower() + name[1:]
-        return getattr(self, normalized_name, self._properties.get(name, None))
+        if default_value is None:
+            normalized_name = name[0].lower() + name[1:]
+            default_value = getattr(self, normalized_name, None)
+        return self._properties.get(name, default_value)
 
     def set_property(self, name, value, persist_changes=True):
         """Sets property value
@@ -93,8 +108,9 @@ class ClientObject(object):
         :param any value: Property value
         :param bool persist_changes: Persist changes
         """
+        self._properties_metadata[name] = {}
         if persist_changes:
-            self._changed_properties.append(name)
+            self.set_metadata(name, "persist", True)
 
         prop_type = self.get_property(name)
         if isinstance(prop_type, ClientObject) or isinstance(prop_type, ClientValue) and value is not None:
@@ -110,32 +126,31 @@ class ClientObject(object):
             self._properties[name] = value
         return self
 
-    def ensure_property(self, name, action=None):
+    def ensure_property(self, name, action, *args, **kwargs):
         """
         Ensures if property is loaded
 
         :type action: () -> None
         :type name: str
         """
-        return self.ensure_properties([name], action)
+        return self.ensure_properties([name], action, *args, **kwargs)
 
-    def ensure_properties(self, names, action):
+    def ensure_properties(self, names, action, *args, **kwargs):
         """
         Ensure if list of properties are loaded
 
-        :type action: () -> None
+        :type action: (any) -> None
         :type names: str or list[str]
         """
-        def _exec_action():
-            if callable(action):
-                action()
 
         names_to_include = [n for n in names if not self.is_property_available(n)]
         if len(names_to_include) > 0:
-            qry = self.context.load(self, names_to_include)
-            self.context.execute_after_query(qry, _exec_action)
+            from office365.runtime.queries.read_entity_query import ReadEntityQuery
+            qry = ReadEntityQuery(self, names_to_include)
+            self.context.add_query(qry, set_as_current=False)
+            self.context.after_query_execute(qry, action, *args, **kwargs)
         else:
-            _exec_action()
+            action(*args, **kwargs)
         return self
 
     def clone_object(self):
@@ -189,11 +204,15 @@ class ClientObject(object):
         """
         :type json_format: office365.runtime.odata.odata_json_format.ODataJsonFormat or None
         """
-        json = dict((k, self.get_property(k)) for k in self.properties if k in self._changed_properties)
+        ser_prop_names = [n for n, p in self._properties_metadata.items() if p.get("persist", False) is True]
+        json = dict((k, self.get_property(k)) for k in self.properties if k in ser_prop_names)
         for k, v in json.items():
             if isinstance(v, ClientObject) or isinstance(v, ClientValue):
                 json[k] = v.to_json(json_format)
 
-        if isinstance(json_format, JsonLightFormat) and json_format.is_verbose and self.entity_type_name is not None:
-            json[json_format.metadata_type_tag_name] = {'type': self.entity_type_name}
+        if json and self.entity_type_name is not None:
+            if isinstance(json_format, JsonLightFormat) and json_format.is_verbose:
+                json[json_format.metadata_type_tag_name] = {'type': self.entity_type_name}
+            elif isinstance(json_format, ODataJsonFormat) and json_format.metadata == "minimal":
+                json[json_format.metadata_type_tag_name] = "#" + self.entity_type_name
         return json
