@@ -2,8 +2,8 @@ import os
 
 from office365.runtime.queries.service_operation_query import ServiceOperationQuery
 from office365.runtime.resource_path_service_operation import ResourcePathServiceOperation
-from office365.sharepoint.actions.create_file import create_file_query
-from office365.sharepoint.actions.upload_session import UploadSessionQuery
+from office365.sharepoint.internal.create_file import create_file_query
+from office365.sharepoint.internal.upload_session import UploadSessionQuery
 from office365.sharepoint.base_entity_collection import BaseEntityCollection
 from office365.sharepoint.files.file import File
 from office365.sharepoint.files.file_creation_information import FileCreationInformation
@@ -22,6 +22,18 @@ class FileCollection(BaseEntityCollection):
         """
         return super(FileCollection, self).get()
 
+    def upload(self, file_name, content):
+        """Uploads a file into folder
+
+        :type file_name: str
+        :type content: bytes or str
+        :rtype: office365.sharepoint.files.file.File
+        """
+        info = FileCreationInformation(url=file_name, overwrite=True, content=content)
+        qry = create_file_query(self, info)
+        self.context.add_query(qry)
+        return qry.return_type
+
     def create_upload_session(self, source_path, chunk_size, chunk_uploaded=None, *chunk_func_args):
         """Upload a file as multiple chunks
 
@@ -35,14 +47,11 @@ class FileCollection(BaseEntityCollection):
         if file_size > chunk_size:
             qry = UploadSessionQuery(self, source_path, chunk_size, chunk_uploaded, chunk_func_args)
             self.context.add_query(qry)
-            return qry.file
+            return qry.return_type
         else:
             with open(source_path, 'rb') as content_file:
                 file_content = content_file.read()
-            info = FileCreationInformation(url=os.path.basename(source_path), overwrite=True, content=file_content)
-            qry = create_file_query(self, info)
-            self.context.add_query(qry)
-            return qry.return_type
+            return self.upload(os.path.basename(source_path), file_content)
 
     def add(self, file_creation_information):
         """Creates a File resource
