@@ -3,13 +3,16 @@ from office365.runtime.client_value_collection import ClientValueCollection
 from office365.runtime.queries.service_operation_query import ServiceOperationQuery
 from office365.runtime.resource_path import ResourcePath
 from office365.runtime.resource_path_service_operation import ResourcePathServiceOperation
+from office365.sharepoint.base_entity_collection import BaseEntityCollection
 from office365.sharepoint.changes.change_collection import ChangeCollection
 from office365.sharepoint.changes.change_query import ChangeQuery
 from office365.sharepoint.comments.comment_collection import CommentCollection
 from office365.sharepoint.fields.field_lookup_value import FieldLookupValue
 from office365.sharepoint.fields.fieldMultiLookupValue import FieldMultiLookupValue
 from office365.sharepoint.likes.likedByInformation import LikedByInformation
+from office365.sharepoint.listitems.list_item_version import ListItemVersion
 from office365.sharepoint.permissions.securable_object import SecurableObject
+from office365.sharepoint.reputationmodel.reputation import Reputation
 from office365.sharepoint.sharing.externalSharingSiteOption import ExternalSharingSiteOption
 from office365.sharepoint.sharing.object_sharing_information import ObjectSharingInformation
 from office365.sharepoint.sharing.sharing_result import SharingResult
@@ -32,7 +35,29 @@ class ListItem(SecurableObject):
         """
         super(ListItem, self).__init__(context, resource_path)
         if parent_list is not None:
-            self.properties["ParentList"] = parent_list
+            self.set_property("ParentList", parent_list, False)
+
+    def set_rating(self, value):
+        """
+        :type value: int
+        """
+        return_value = ClientResult(self.context)
+
+        def _list_item_loaded():
+            Reputation.set_rating(self.context, self.parent_list.id, self.id, value, return_value)
+        self.parent_list.ensure_properties(["Id", "ParentList"], _list_item_loaded)
+        return return_value
+
+    def set_like(self, value):
+        """
+        :type value: bool
+        """
+        return_value = ClientResult(self.context)
+
+        def _list_item_loaded():
+            Reputation.set_like(self.context, self.parent_list.id, self.id, value, return_value)
+        self.parent_list.ensure_properties(["Id", "ParentList"], _list_item_loaded)
+        return return_value
 
     def get_wopi_frame_url(self, action):
         """
@@ -67,16 +92,16 @@ class ListItem(SecurableObject):
         return changes
 
     def share(self, user_principal_name,
-              shareOption=ExternalSharingSiteOption.View,
-              sendEmail=True, emailSubject=None, emailBody=None):
+              share_option=ExternalSharingSiteOption.View,
+              send_email=True, email_subject=None, email_body=None):
         """
         Share a ListItem (file or folder facet)
 
         :param str user_principal_name: User identifier
-        :param ExternalSharingSiteOption shareOption: The sharing type of permission to grant on the object.
-        :param bool sendEmail: A flag to determine if an email notification SHOULD be sent (if email is configured).
-        :param str emailSubject: The email subject.
-        :param str emailBody: The email subject.
+        :param ExternalSharingSiteOption share_option: The sharing type of permission to grant on the object.
+        :param bool send_email: A flag to determine if an email notification SHOULD be sent (if email is configured).
+        :param str email_subject: The email subject.
+        :param str email_body: The email subject.
         :rtype: SharingResult
         """
 
@@ -93,9 +118,9 @@ class ListItem(SecurableObject):
 
         def _picker_value_resolved(picker_value):
             from office365.sharepoint.webs.web import Web
-            result.value = Web.share_object(self.context, file_result.value, picker_value, role_values[shareOption],
+            result.value = Web.share_object(self.context, file_result.value, picker_value, role_values[share_option],
                                             0,
-                                            False, sendEmail, False, emailSubject, emailBody)
+                                            False, send_email, False, email_subject, email_body)
 
         self.ensure_property("EncodedAbsUrl", _property_resolved)
         params = ClientPeoplePickerQueryParameters(user_principal_name)
@@ -178,20 +203,20 @@ class ListItem(SecurableObject):
         self.context.add_query(qry)
         return comments
 
-    def parse_and_set_field_value(self, fieldName, value):
+    def parse_and_set_field_value(self, field_name, value):
         """Sets the value of the field (2) for the list item based on an implementation-specific transformation
            of the value..
-           :param str fieldName: Specifies the field internal name.
+           :param str field_name: Specifies the field internal name.
            :param str value: Specifies the new value for the field (2).
 
         """
         payload = {
-            "fieldName": fieldName,
+            "fieldName": field_name,
             "value": value
         }
-        qry = ServiceOperationQuery(self,
-                                    "ParseAndSetFieldValue", None, payload, None, None)
+        qry = ServiceOperationQuery(self, "ParseAndSetFieldValue", None, payload)
         self.context.add_query(qry)
+        return self
 
     @property
     def display_name(self):
@@ -239,8 +264,7 @@ class ListItem(SecurableObject):
         """Gets a value that specifies the effective permissions on the list item that are assigned
            to the current user."""
         from office365.sharepoint.permissions.base_permissions import BasePermissions
-        return self.properties.get("EffectiveBasePermissions",
-                                   BasePermissions())
+        return self.properties.get("EffectiveBasePermissions", BasePermissions())
 
     @property
     def field_values(self):
@@ -274,35 +298,40 @@ class ListItem(SecurableObject):
     def liked_by_information(self):
         """
         Gets a value that specifies the list item identifier.
-        :rtype: int
+        :rtype: LikedByInformation
         """
         return self.properties.get("LikedByInformation",
                                    LikedByInformation(self.context,
                                                       ResourcePath("likedByInformation", self.resource_path)))
 
-    def get_property(self, name):
-        if name == "ContentType":
-            return self.content_type
-        elif name == "ParentList":
-            return self.parent_list
-        elif name == "EffectiveBasePermissions":
-            return self.effective_base_permissions
-        elif name == "AttachmentFiles":
-            return self.attachment_files
-        elif name == "LikedByInformation":
-            return self.liked_by_information
-        else:
-            value = super(ListItem, self).get_property(name)
-            if self.is_property_available(name[:-2]):
-                lookup_value = super(ListItem, self).get_property(name[:-2])
-                if isinstance(lookup_value, FieldMultiLookupValue):
-                    return ClientValueCollection(int, [v.LookupId for v in lookup_value])
-                elif isinstance(lookup_value, FieldLookupValue):
-                    return lookup_value.LookupId
-            return value
+    @property
+    def versions(self):
+        """Gets the collection of item version objects that represent the versions of the item."""
+        return self.properties.get('Versions',
+                                   BaseEntityCollection(self.context, ListItemVersion,
+                                                        ResourcePath("versions", self.resource_path)))
+
+    def get_property(self, name, default_value=None):
+        if default_value is None:
+            property_mapping = {
+                "AttachmentFiles": self.attachment_files,
+                "ContentType": self.content_type,
+                "EffectiveBasePermissions": self.effective_base_permissions,
+                "LikedByInformation": self.liked_by_information,
+                "ParentList": self.parent_list,
+            }
+            default_value = property_mapping.get(name, None)
+
+        value = super(ListItem, self).get_property(name, default_value)
+        if self.is_property_available(name[:-2]):
+            lookup_value = super(ListItem, self).get_property(name[:-2], default_value)
+            if isinstance(lookup_value, FieldMultiLookupValue):
+                return ClientValueCollection(int, [v.LookupId for v in lookup_value])
+            elif isinstance(lookup_value, FieldLookupValue):
+                return lookup_value.LookupId
+        return value
 
     def set_property(self, name, value, persist_changes=True):
-
         if persist_changes:
             if isinstance(value, TaxonomyFieldValueCollection):
                 self._set_taxonomy_field_value(name, value)
@@ -332,7 +361,7 @@ class ListItem(SecurableObject):
             tax_text_field = self.parent_list.fields.get_by_id(tax_field.properties["TextField"])
 
             def _tax_text_field_loaded():
-                self.set_property(tax_text_field.properties["StaticName"], str(value)).update()
+                self.set_property(tax_text_field.properties["StaticName"], str(value))
 
             tax_text_field.ensure_property("StaticName", _tax_text_field_loaded)
 
@@ -342,7 +371,7 @@ class ListItem(SecurableObject):
         """
         Determine metadata annotation for ListItem entity
 
-        :param office365.sharepoint.lists.list.List target_list: List entity
+        :param office365.sharepoint.lists.list.List target_list: List resource
         """
 
         def _init_item_type():
