@@ -12,6 +12,7 @@ from office365.sharepoint.fields.field_collection import FieldCollection
 from office365.sharepoint.fields.related_field_collection import RelatedFieldCollection
 from office365.sharepoint.files.checkedOutFileCollection import CheckedOutFileCollection
 from office365.sharepoint.files.file import File
+from office365.sharepoint.flows.flow_synchronization_result import FlowSynchronizationResult
 from office365.sharepoint.folders.folder import Folder
 from office365.sharepoint.forms.form_collection import FormCollection
 from office365.sharepoint.listitems.caml.caml_query import CamlQuery
@@ -19,6 +20,7 @@ from office365.sharepoint.listitems.creation_information_using_path import ListI
 from office365.sharepoint.listitems.form_update_value import ListItemFormUpdateValue
 from office365.sharepoint.listitems.listitem import ListItem
 from office365.sharepoint.listitems.listItem_collection import ListItemCollection
+from office365.sharepoint.lists.list_rule import SPListRule
 from office365.sharepoint.pages.wiki_page_creation_information import WikiPageCreationInformation
 from office365.sharepoint.permissions.securable_object import SecurableObject
 from office365.sharepoint.usercustomactions.user_custom_action_collection import UserCustomActionCollection
@@ -34,6 +36,70 @@ class List(SecurableObject):
     def __init__(self, context, resource_path=None):
         super(List, self).__init__(context, resource_path)
 
+    def get_all_rules(self):
+        return_type = ClientResult(self.context, ClientValueCollection(SPListRule))
+        qry = ServiceOperationQuery(self, "GetAllRules", None, None, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
+    def get_business_app_operation_status(self):
+        return_type = FlowSynchronizationResult(self.context)
+        qry = ServiceOperationQuery(self, "GetBusinessAppOperationStatus", None, None, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
+    def sync_flow_callback_url(self, flow_id):
+        """
+        :param str flow_id:
+        """
+        return_type = FlowSynchronizationResult(self.context)
+        qry = ServiceOperationQuery(self, "SyncFlowCallbackUrl", None, {"flowId": flow_id}, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
+    def sync_flow_instance(self, flow_id):
+        """
+        :param str flow_id:
+        """
+        return_type = FlowSynchronizationResult(self.context)
+        qry = ServiceOperationQuery(self, "SyncFlowInstance", None, {"flowId": flow_id}, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
+    def sync_flow_instances(self, retrieve_group_flows):
+        """
+        :param bool retrieve_group_flows:
+        """
+        return_type = FlowSynchronizationResult(self.context)
+        payload = {"retrieveGroupFlows": retrieve_group_flows}
+        qry = ServiceOperationQuery(self, "SyncFlowInstances", None, payload, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
+    def sync_flow_templates(self, category):
+        """
+        :param str category:
+        """
+        return_type = FlowSynchronizationResult(self.context)
+        payload = {"category": category}
+        qry = ServiceOperationQuery(self, "SyncFlowTemplates", None, payload, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
+    def create_document_with_default_name(self, folder_path, extension):
+        """
+        :param str folder_path:
+        :param str extension:
+        """
+        return_type = FlowSynchronizationResult(self.context)
+        payload = {
+            "folderPath": folder_path,
+            "extension": extension
+        }
+        qry = ServiceOperationQuery(self, "CreateDocumentWithDefaultName", None, payload, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
     def recycle(self):
         """Moves the list to the Recycle Bin and returns the identifier of the new Recycle Bin item."""
         result = ClientResult(self.context)
@@ -41,14 +107,14 @@ class List(SecurableObject):
         self.context.add_query(qry)
         return result
 
-    def render_list_data(self, viewXml):
+    def render_list_data(self, view_xml):
         """
 
-        :param str viewXml: View xml
+        :param str view_xml: View xml
         """
         result = ClientResult(self.context)
         payload = {
-            "viewXml": viewXml
+            "viewXml": view_xml
         }
         qry = ServiceOperationQuery(self, "RenderListData", None, payload, None, result)
         self.context.add_query(qry)
@@ -94,11 +160,14 @@ class List(SecurableObject):
         self.context.add_query(qry)
         return result
 
-    def get_lookup_field_choices(self, targetFieldName, pagingInfo=None):
+    def get_lookup_field_choices(self, target_field_name, paging_info=None):
+        """
+
+        """
         result = ClientResult(self.context)
         params = {
-            "targetFieldName": targetFieldName,
-            "pagingInfo": pagingInfo
+            "targetFieldName": target_field_name,
+            "pagingInfo": paging_info
         }
         qry = ServiceOperationQuery(self, "GetLookupFieldChoices", params, None, None, result)
         self.context.add_query(qry)
@@ -114,34 +183,34 @@ class List(SecurableObject):
         self.context.add_query(qry)
         return result
 
-    def save_as_template(self, fileName, name, description, saveData):
+    def save_as_template(self, file_name, name, description, save_data):
         """
         Saves the list as a template in the list template gallery and includes the option of saving with or
         without the data that is contained in the current list.
 
-        :param bool saveData: true to save the data of the original list along with the list template; otherwise, false.
+        :param bool save_data: true to save the data of the original list along with the list template; otherwise, false
         :param str description: A string that contains the description for the list template.
         :param str name: A string that contains the title for the list template.
-        :param str fileName: A string that contains the file name for the list template with an .stp extension.
+        :param str file_name: A string that contains the file name for the list template with an .stp extension.
         :return:
         """
         payload = {
-            "strFileName": fileName,
+            "strFileName": file_name,
             "strName": name,
             "strDescription": description,
-            "bSaveData": saveData
+            "bSaveData": save_data
         }
         qry = ServiceOperationQuery(self, "saveAsTemplate", None, payload, None, None)
         self.context.add_query(qry)
         return self
 
-    def get_item_by_unique_id(self, uniqueId):
+    def get_item_by_unique_id(self, unique_id):
         """
         Returns the list item with the specified ID.
 
-        :param str uniqueId:"""
-        item = ListItem(self.context, ResourcePathServiceOperation("getItemByUniqueId", [uniqueId], self.resource_path))
-        return item
+        :param str unique_id:"""
+        return ListItem(self.context,
+                        ResourcePathServiceOperation("getItemByUniqueId", [unique_id], self.resource_path))
 
     def get_web_dav_url(self, source_url):
         """
@@ -243,8 +312,7 @@ class List(SecurableObject):
 
         :type view_id: str
         """
-        view = View(self.context, ResourcePathServiceOperation("getView", [view_id], self.resource_path), self)
-        return view
+        return View(self.context, ResourcePathServiceOperation("getView", [view_id], self.resource_path), self)
 
     def get_changes(self, query=None):
         """Returns the collection of changes from the change log that have occurred within the list,
@@ -265,6 +333,12 @@ class List(SecurableObject):
         self.context.add_query(qry)
         return result
 
+    def reserve_list_item_id(self):
+        result = ClientResult(self.context)
+        qry = ServiceOperationQuery(self, "ReserveListItemId", None, None, None, result)
+        self.context.add_query(qry)
+        return result
+
     def get_related_fields(self):
         """Returns a collection of lookup fields that use this list as a data source and
             that have FieldLookup.IsRelationship set to true.
@@ -272,6 +346,14 @@ class List(SecurableObject):
         return RelatedFieldCollection(self.context,
                                       ResourcePathServiceOperation("getRelatedFields", [], self.resource_path))
 
+    @property
+    def id(self):
+        """
+        Gets a value that specifies the list identifier.
+        :rtype: str
+        """
+        return self.properties.get("Id", None)
+
     @property
     def enable_folder_creation(self):
         """
@@ -317,7 +399,7 @@ class List(SecurableObject):
         """Gets one or more webhook subscriptions on a SharePoint list."""
         return self.properties.get('Subscriptions',
                                    SubscriptionCollection(self.context,
-                                                          ResourcePath("Subscriptions", self.resource_path)))
+                                                          ResourcePath("Subscriptions", self.resource_path), self))
 
     @property
     def views(self):
@@ -407,19 +489,19 @@ class List(SecurableObject):
     def parent_web_path(self):
         return self.properties.get('ParentWebPath', None)
 
-    def get_property(self, name):
-        if name == "UserCustomActions":
-            return self.user_custom_actions
-        elif name == "ParentWeb":
-            return self.parent_web
-        elif name == "RootFolder":
-            return self.root_folder
-        elif name == "ContentTypes":
-            return self.content_types
-        elif name == "DefaultView":
-            return self.default_view
-        else:
-            return super(List, self).get_property(name)
+    def get_property(self, name, default_value=None):
+        if default_value is None:
+            property_mapping = {
+                "ContentTypes": self.content_types,
+                "CustomActionElements": self.custom_action_elements,
+                "DefaultView": self.default_view,
+                "EventReceivers": self.event_receivers,
+                "ParentWeb": self.parent_web,
+                "RootFolder": self.root_folder,
+                "UserCustomActions": self.user_custom_actions
+            }
+            default_value = property_mapping.get(name, None)
+        return super(List, self).get_property(name, default_value)
 
     def set_property(self, name, value, persist_changes=True):
         super(List, self).set_property(name, value, persist_changes)
