@@ -5,8 +5,10 @@ from office365.runtime.queries.service_operation_query import ServiceOperationQu
 from office365.runtime.resource_path import ResourcePath
 from office365.runtime.resource_path_service_operation import ResourcePathServiceOperation
 from office365.sharepoint.alerts.alert_collection import AlertCollection
+from office365.sharepoint.base_entity_collection import BaseEntityCollection
 from office365.sharepoint.changes.change_collection import ChangeCollection
-from office365.sharepoint.clientsidecomponent.types import SPClientSideComponentQueryResult
+from office365.sharepoint.clientsidecomponent.types import SPClientSideComponentQueryResult, \
+    SPClientSideComponentIdentifier
 from office365.sharepoint.contenttypes.content_type_collection import ContentTypeCollection
 from office365.sharepoint.eventreceivers.event_receiver_definition import EventReceiverDefinitionCollection
 from office365.sharepoint.fields.field_collection import FieldCollection
@@ -27,6 +29,7 @@ from office365.sharepoint.principal.group import Group
 from office365.sharepoint.principal.group_collection import GroupCollection
 from office365.sharepoint.principal.user import User
 from office365.sharepoint.principal.user_collection import UserCollection
+from office365.sharepoint.pushnotifications.push_notification_subscriber import PushNotificationSubscriber
 from office365.sharepoint.recyclebin.recycleBinItemCollection import RecycleBinItemCollection
 from office365.sharepoint.sharing.externalSharingSiteOption import ExternalSharingSiteOption
 from office365.sharepoint.sharing.objectSharingSettings import ObjectSharingSettings
@@ -38,6 +41,7 @@ from office365.sharepoint.ui.applicationpages.client_people_picker import (
     ClientPeoplePickerWebServiceInterface, ClientPeoplePickerQueryParameters
 )
 from office365.sharepoint.webparts.client_web_part_collection import ClientWebPartCollection
+from office365.sharepoint.webs.context_web_information import ContextWebInformation
 from office365.sharepoint.webs.regional_settings import RegionalSettings
 from office365.sharepoint.webs.web_information_collection import WebInformationCollection
 from office365.sharepoint.webs.web_template_collection import WebTemplateCollection
@@ -59,6 +63,49 @@ class Web(SecurableObject):
         super(Web, self).__init__(context, resource_path)
         self._web_url = None
 
+    def get_push_notification_subscriber(self, device_app_instance_id):
+        return_type = PushNotificationSubscriber(self.context)
+        qry = ServiceOperationQuery(self, "GetPushNotificationSubscriber", [device_app_instance_id], None,
+                                    None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
+    def get_client_side_components(self, components):
+        """
+        Returns the client side components for the requested components.
+        Client components include data necessary to render Client Side Web Parts and Client Side Applications.
+
+        :param list components: array of requested components, defined by id and version.
+        """
+        return_type = ClientResult(self.context, ClientValueCollection(SPClientSideComponentIdentifier))
+        payload = {
+            "components": components
+        }
+        qry = ServiceOperationQuery(self, "GetClientSideComponents", None, payload,
+                                    None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
+    def get_push_notification_subscribers_by_user(self, user_or_username):
+        """
+
+        :param str or User user_or_username:
+        """
+        return_type = BaseEntityCollection(self.context, PushNotificationSubscriber)
+
+        if isinstance(user_or_username, User):
+            def _user_loaded():
+                next_qry = ServiceOperationQuery(self, "GetPushNotificationSubscribersByUser",
+                                                 [user_or_username.login_name], None, None, return_type)
+                self.context.add_query(next_qry)
+
+            user_or_username.ensure_property("LoginName", _user_loaded)
+        else:
+            qry = ServiceOperationQuery(self, "GetPushNotificationSubscribersByUser", [user_or_username], None,
+                                        None, return_type)
+            self.context.add_query(qry)
+        return return_type
+
     @staticmethod
     def create_organization_sharing_link(context, url, is_edit_link):
         """
@@ -74,6 +121,19 @@ class Web(SecurableObject):
         context.add_query(qry)
         return result
 
+    @staticmethod
+    def get_context_web_information(context):
+        """
+        Returns an object that specifies metadata about the site
+
+        :type context: office365.sharepoint.client_context.ClientContext
+        """
+        result = ClientResult(context, ContextWebInformation())
+        qry = ServiceOperationQuery(context.web, "GetContextWebInformation", None, None, None, result)
+        qry.static = True
+        context.add_query(qry)
+        return result
+
     @staticmethod
     def get_web_url_from_page_url(context, page_full_url):
         """Gets Web from page url
@@ -157,19 +217,23 @@ class Web(SecurableObject):
         self.context.add_query(qry)
         return users
 
-    def get_recycle_bin_items(self, pagingInfo=None, rowLimit=100, isAscending=True, orderBy=None, itemState=None):
+    def get_recycle_bin_items(self, paging_info=None, row_limit=100, is_ascending=True, order_by=None, item_state=None):
         """
+        Gets the recycle bin items that are based on the specified query.
 
-        :param str pagingInfo:
-        :param int rowLimit:
-        :param bool isAscending:
-        :param orderBy: int
-        :param int itemState:
+        :param str paging_info: an Object that is used to obtain the next set of rows in a paged view of the Recycle Bin
+        :param int row_limit: a limit for the number of items returned in the query per page.
+        :param bool is_ascending: a Boolean value that specifies whether to sort in ascending order.
+        :param int order_by: the column by which to order the Recycle Bin query.
+        :param int item_state: Recycle Bin stage of items to return in the query.
         """
         result = RecycleBinItemCollection(self.context)
         payload = {
-            "rowLimit": rowLimit,
-            "isAscending": isAscending
+            "rowLimit": row_limit,
+            "isAscending": is_ascending,
+            "pagingInfo": paging_info,
+            "orderBy": order_by,
+            "itemState": item_state
         }
         qry = ServiceOperationQuery(self, "GetRecycleBinItems", None, payload, None, result)
         self.context.add_query(qry)
@@ -206,12 +270,12 @@ class Web(SecurableObject):
         """
         return_list = List(self.context)
         self.lists.add_child(return_list)
-        from office365.sharepoint.types.resource_path import ResourcePath as SPResPath
         qry = ServiceOperationQuery(self, "GetListUsingPath", SPResPath(decoded_url), None, None, return_list)
         self.context.add_query(qry)
         return return_list
 
     def get_regional_datetime_schema(self):
+        """Get DateTime Schema based on regional settings"""
         return_type = ClientResult(self.context)
         qry = ServiceOperationQuery(self, "GetRegionalDateTimeSchema", None, None, None, return_type)
         self.context.add_query(qry)
@@ -219,7 +283,10 @@ class Web(SecurableObject):
 
     def get_sharing_link_data(self, link_url):
         """
-        :type link_url: str
+        This method determines basic information about the supplied link URL, including limited data about the object
+        the link URL refers to and any additional sharing link data if the link URL is a tokenized sharing link
+
+        :param str link_url: A URL that is either a tokenized sharing link or a canonical URL for a document
         """
         result = SharingLinkData()
         qry = ServiceOperationQuery(self, "GetSharingLinkData", [link_url], None, None, result)
@@ -339,11 +406,12 @@ class Web(SecurableObject):
     def ensure_user(self, login_name):
         """Checks whether the specified logon name belongs to a valid user of the website, and if the logon name does
         not already exist, adds it to the website.
+
         :type login_name: str
         """
         target_user = User(self.context)
-        self.siteUsers.add_child(target_user)
-        qry = ServiceOperationQuery(self, "ensureUser", [login_name], None, None, target_user)
+        self.site_users.add_child(target_user)
+        qry = ServiceOperationQuery(self, "EnsureUser", [login_name], None, None, target_user)
         self.context.add_query(qry)
         return target_user
 
@@ -362,7 +430,17 @@ class Web(SecurableObject):
         :type permission_mask: BasePermissions
         """
         result = ClientResult(self.context)
-        qry = ServiceOperationQuery(self, "doesUserHavePermissions", permission_mask, None, None, result)
+        qry = ServiceOperationQuery(self, "DoesUserHavePermissions", permission_mask, None, None, result)
+        self.context.add_query(qry)
+        return result
+
+    def does_push_notification_subscriber_exist(self, device_app_instance_id):
+        """
+        :type device_app_instance_id: str
+        """
+        result = ClientResult(self.context)
+        params = {"deviceAppInstanceId": device_app_instance_id}
+        qry = ServiceOperationQuery(self, "DoesPushNotificationSubscriberExist", params, None, None, result)
         self.context.add_query(qry)
         return result
 
@@ -372,7 +450,7 @@ class Web(SecurableObject):
         :type unique_id: str
         """
         folder = Folder(self.context)
-        qry = ServiceOperationQuery(self, "getFolderById", [unique_id], None, None, folder)
+        qry = ServiceOperationQuery(self, "GetFolderById", [unique_id], None, None, folder)
         self.context.add_query(qry)
         return folder
 
@@ -408,17 +486,17 @@ class Web(SecurableObject):
         self.context.add_query(qry)
         return changes
 
-    def get_available_web_templates(self, lcid=1033, doIncludeCrossLanguage=False):
+    def get_available_web_templates(self, lcid=1033, do_include_cross_language=False):
         """
         Returns a collection of site templates available for the site.
 
         :param int lcid: Specifies the LCID of the site templates to be retrieved.
-        :param bool doIncludeCrossLanguage: Specifies whether to include language-neutral site templates.
+        :param bool do_include_cross_language: Specifies whether to include language-neutral site templates.
         :return:
         """
         params = {
             "lcid": lcid,
-            "doIncludeCrossLanguage": doIncludeCrossLanguage
+            "doIncludeCrossLanguage": do_include_cross_language
         }
         return_type = WebTemplateCollection(self.context,
                                             ResourcePathServiceOperation("GetAvailableWebTemplates ", params,
@@ -428,14 +506,14 @@ class Web(SecurableObject):
         self.context.add_query(qry)
         return return_type
 
-    def apply_web_template(self, webTemplate):
+    def apply_web_template(self, web_template):
         """
         Applies the specified site definition or site template to the website that has no template applied to it.
 
-        :param str webTemplate: The name of the site definition or the file name of the site template to be applied.
+        :param str web_template: The name of the site definition or the file name of the site template to be applied.
         :return:
         """
-        qry = ServiceOperationQuery(self, "ApplyWebTemplate", {"webTemplate": webTemplate})
+        qry = ServiceOperationQuery(self, "ApplyWebTemplate", {"webTemplate": web_template})
         self.context.add_query(qry)
         return self
 
@@ -449,35 +527,44 @@ class Web(SecurableObject):
         self.context.add_query(qry)
         return return_type
 
-    def get_file_by_guest_url(self, guestUrl):
+    def get_file_by_guest_url(self, guest_url):
         """
-        :type guestUrl: str
+        :type guest_url: str
         """
         return_type = File(self.context)
-        qry = ServiceOperationQuery(self, "GetFileByGuestUrl", [guestUrl], None, None, return_type)
+        qry = ServiceOperationQuery(self, "GetFileByGuestUrl", [guest_url], None, None, return_type)
         self.context.add_query(qry)
         return return_type
 
-    def get_folder_by_guest_url(self, guestUrl):
+    def get_file_by_wopi_frame_url(self, wopi_frame_url):
         """
-        :type guestUrl: str
+        :param str wopi_frame_url:
         """
         return_type = File(self.context)
-        qry = ServiceOperationQuery(self, "GetFolderByGuestUrl", [guestUrl], None, None, return_type)
+        qry = ServiceOperationQuery(self, "GetFileByWOPIFrameUrl", [wopi_frame_url], None, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
+    def get_folder_by_guest_url(self, guest_url):
+        """
+        :type guest_url: str
+        """
+        return_type = File(self.context)
+        qry = ServiceOperationQuery(self, "GetFolderByGuestUrl", [guest_url], None, None, return_type)
         self.context.add_query(qry)
         return return_type
 
     def share(self, user_principal_name,
-              shareOption=ExternalSharingSiteOption.View,
-              sendEmail=True, emailSubject=None, emailBody=None):
+              share_option=ExternalSharingSiteOption.View,
+              send_email=True, email_subject=None, email_body=None):
         """
         Share a Web with user
 
         :param str user_principal_name: User identifier
-        :param ExternalSharingSiteOption shareOption: The sharing type of permission to grant on the object.
-        :param bool sendEmail: A flag to determine if an email notification SHOULD be sent (if email is configured).
-        :param str emailSubject: The email subject.
-        :param str emailBody: The email subject.
+        :param ExternalSharingSiteOption share_option: The sharing type of permission to grant on the object.
+        :param bool send_email: A flag to determine if an email notification SHOULD be sent (if email is configured).
+        :param str email_subject: The email subject.
+        :param str email_body: The email subject.
         :rtype: SharingResult
         """
 
@@ -491,14 +578,14 @@ class Web(SecurableObject):
             def _web_loaded():
                 sharing_result.value = Web.share_object(self.context, self.url, picker_result.value, role_value,
                                                         0,
-                                                        False, sendEmail, False, emailSubject, emailBody)
+                                                        False, send_email, False, email_subject, email_body)
 
             self.ensure_property("Url", _web_loaded)
 
         params = ClientPeoplePickerQueryParameters(user_principal_name)
         ClientPeoplePickerWebServiceInterface.client_people_picker_resolve_user(self.context, params,
                                                                                 _picker_value_resolved)
-        Web._resolve_group_value(self.context, shareOption, _grp_resolved)
+        Web._resolve_group_value(self.context, share_option, _grp_resolved)
         return sharing_result.value
 
     def unshare(self):
@@ -576,21 +663,21 @@ class Web(SecurableObject):
         context.after_execute(_group_resolved)
 
     @staticmethod
-    def get_sharing_link_kind(context, fileUrl):
+    def get_sharing_link_kind(context, file_url):
         """
 
         :param office365.sharepoint.client_context.ClientContext context:
-        :param str fileUrl:
+        :param str file_url:
         """
         result = ClientResult(context)
-        qry = ServiceOperationQuery(context.web, "GetSharingLinkKind", None, {"fileUrl": fileUrl}, None, result)
+        qry = ServiceOperationQuery(context.web, "GetSharingLinkKind", None, {"fileUrl": file_url}, None, result)
         qry.static = True
         context.add_query(qry)
         return result
 
     @staticmethod
-    def share_object(context, url, peoplePickerInput,
-                     roleValue=None,
+    def share_object(context, url, people_picker_input,
+                     role_value=None,
                      groupId=0, propagateAcl=False,
                      sendEmail=True, includeAnonymousLinkInEmail=False, emailSubject=None, emailBody=None,
                      useSimplifiedRoles=True):
@@ -601,8 +688,8 @@ class Web(SecurableObject):
 
         :param office365.sharepoint.client_context.ClientContext context: SharePoint context
         :param str url: The URL of the website with the path of an object in SharePoint query string parameters.
-        :param str roleValue: The sharing role value for the type of permission to grant on the object.
-        :param str peoplePickerInput: A string of JSON representing users in people picker format.
+        :param str role_value: The sharing role value for the type of permission to grant on the object.
+        :param str people_picker_input: A string of JSON representing users in people picker format.
         :param int groupId: The ID of the group to be added. Zero if not adding to a permissions group.
         :param bool propagateAcl:  A flag to determine if permissions SHOULD be pushed to items with unique permissions.
         :param bool sendEmail: A flag to determine if an email notification SHOULD be sent (if email is configured).
@@ -618,8 +705,8 @@ class Web(SecurableObject):
         payload = {
             "url": url,
             "groupId": groupId,
-            "peoplePickerInput": peoplePickerInput,
-            "roleValue": roleValue,
+            "peoplePickerInput": people_picker_input,
+            "roleValue": role_value,
             "includeAnonymousLinkInEmail": includeAnonymousLinkInEmail,
             "propagateAcl": propagateAcl,
             "sendEmail": sendEmail,
@@ -668,8 +755,7 @@ class Web(SecurableObject):
         for example, "/sites/MySite/Shared Documents/MyDocument.docx".
         :return: ListItem
         """
-        return_item = ListItem(self.context, ResourcePathServiceOperation("GetListItem", [str_url], self.resource_path))
-        return return_item
+        return ListItem(self.context, ResourcePathServiceOperation("GetListItem", [str_url], self.resource_path))
 
     def get_catalog(self, type_catalog):
         """Gets the list template gallery, site template gallery, or Web Part gallery for the Web site.
@@ -678,6 +764,45 @@ class Web(SecurableObject):
         """
         return List(self.context, ResourcePathServiceOperation("getCatalog", [type_catalog], self.resource_path))
 
+    def page_context_info(self, include_odb_settings, emit_navigation_info):
+        """
+        Return Page context info for the current list being rendered.
+
+        :param bool include_odb_settings:
+        :param bool emit_navigation_info:
+        """
+        return_type = ClientResult(self.context)
+        payload = {
+            "includeODBSettings": include_odb_settings,
+            "emitNavigationInfo": emit_navigation_info
+        }
+        qry = ServiceOperationQuery(self, "PageContextInfo", None, payload, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
+    @property
+    def allow_rss_feeds(self):
+        """Gets a Boolean value that specifies whether the site collection allows RSS feeds.
+
+        :rtype: str
+        """
+        return self.properties.get("AllowRssFeeds", None)
+
+    @property
+    def alternate_css_url(self):
+        """Gets the URL for an alternate cascading style sheet (CSS) to use in the website.
+
+        :rtype: str
+        """
+        return self.properties.get("AlternateCssUrl", None)
+
+    @property
+    def id(self):
+        """
+        :rtype: str
+        """
+        return self.properties.get("Id", None)
+
     @property
     def webs(self):
         """Get child webs"""
@@ -698,13 +823,13 @@ class Web(SecurableObject):
                                    ListCollection(self.context, ResourcePath("lists", self.resource_path)))
 
     @property
-    def siteUsers(self):
+    def site_users(self):
         """Get site users"""
         return self.properties.get('SiteUsers',
                                    UserCollection(self.context, ResourcePath("siteUsers", self.resource_path)))
 
     @property
-    def siteGroups(self):
+    def site_groups(self):
         """Gets the collection of groups for the site collection."""
         return self.properties.get('SiteGroups',
                                    GroupCollection(self.context, ResourcePath("siteGroups", self.resource_path)))
@@ -716,7 +841,7 @@ class Web(SecurableObject):
                                    User(self.context, ResourcePath("CurrentUser", self.resource_path)))
 
     @property
-    def parentWeb(self):
+    def parent_web(self):
         """Gets the parent website of the specified website."""
         return self.properties.get('ParentWeb',
                                    Web(self.context, ResourcePath("ParentWeb", self.resource_path)))
@@ -851,6 +976,13 @@ class Web(SecurableObject):
                                    Navigation(self.context,
                                               ResourcePath("Navigation", self.resource_path)))
 
+    @property
+    def push_notification_subscribers(self):
+        return self.properties.get('PushNotificationSubscribers',
+                                   BaseEntityCollection(self.context, PushNotificationSubscriber,
+                                                        ResourcePath("PushNotificationSubscribers",
+                                                                     self.resource_path)))
+
     @property
     def root_folder(self):
         """Get a root folder"""
@@ -900,16 +1032,20 @@ class Web(SecurableObject):
     def get_property(self, name, default_value=None):
         if default_value is None:
             property_mapping = {
+                "AvailableFields": self.available_fields,
+                "AssociatedOwnerGroup": self.associated_owner_group,
+                "AssociatedMemberGroup": self.associated_member_group,
+                "AssociatedVisitorGroup": self.associated_visitor_group,
                 "ContentTypes": self.content_types,
+                "ClientWebParts": self.client_web_parts,
+                "CurrentUser": self.current_user,
+                "ParentWeb": self.parent_web,
                 "RootFolder": self.root_folder,
                 "RegionalSettings": self.regional_settings,
                 "RoleDefinitions": self.role_definitions,
                 "RecycleBin": self.recycle_bin,
-                "CurrentUser": self.current_user,
-                "AvailableFields": self.available_fields,
-                "AssociatedOwnerGroup": self.associated_owner_group,
-                "AssociatedMemberGroup": self.associated_member_group,
-                "AssociatedVisitorGroup": self.associated_visitor_group
+                "SiteGroups": self.site_groups,
+                "SiteUsers": self.site_users
             }
             default_value = property_mapping.get(name, None)
         return super(Web, self).get_property(name, default_value)
